<?php
namespace Opencart\Catalog\Controller\Extension\ChargeAnywhere\Payment;
class ChargeAnywhere extends \Opencart\System\Engine\Controller {
	private $error = [];
	private $server_capture_refund = array(
		'sandbox' => 'https://webtest.chargeanywhere.com/APIs/PaymentFormAIP.aspx',
		'production' => 'https://www.chargeanywhere.com/APIs/PaymentFormAIP.aspx'
	);	

	private $server_post_payment = array(
		'sandbox' => 'https://webtest.chargeanywhere.com/APIs/PaymentFormSIP.aspx',
		'production' => 'https://www.chargeanywhere.com/APIs/PaymentFormSIP.aspx'
	);	

	
	public function __construct($registry) {
		parent::__construct($registry);

		if (version_compare(phpversion(), '7.1', '>=')) {
			ini_set('precision', 14);
			ini_set('serialize_precision', 14);
		}
	}
	
	public function index(): string {
			$this->load->language('extension/chargeanywhere/payment/chargeanywhere');
		
			$this->load->model('extension/chargeanywhere/payment/chargeanywhere');
			$this->load->model('localisation/country');
			$this->load->model('checkout/order');
				
			$country = $this->model_localisation_country->getCountry($this->config->get('config_country_id'));
			$this->load->model('checkout/cart');
			$totals = [];
			$taxes = $this->cart->getTaxes();
			$total = 0;
			($this->model_checkout_cart->getTotals)($totals, $taxes, $total);
			
			// Setting
			$_config = new \Opencart\System\Engine\Config();
			$_config->addPath(DIR_EXTENSION . 'chargeanywhere/system/config/');
			$_config->load('chargeanywhere');
			$config_setting = $_config->get('chargeanywhere_setting');
			$setting = array_replace_recursive((array)$config_setting, (array)$this->config->get('payment_chargeanywhere_setting'));	
			
			$data['secret'] = $this->config->get('payment_chargeanywhere_secret');
			$data['merchant_id'] = $this->config->get('payment_chargeanywhere_merchant_id');
			$data['terminal_id'] = $this->config->get('payment_chargeanywhere_terminal_id');
			$data['environment'] = $this->config->get('payment_chargeanywhere_environment');
			$data['payment_url'] = $this->server_post_payment[$data['environment']];
			
			$data['transaction_method'] = $this->config->get('payment_chargeanywhere_transaction_method');
			$data['log'] = $this->config->get('payment_chargeanywhere_log');
			$data['accept_credit'] = $this->config->get('payment_chargeanywhere_accept_credit');
			$data['apply_credit'] = $this->config->get('payment_chargeanywhere_apply_credit');
			$data['accept_ach'] = $this->config->get('payment_chargeanywhere_accept_ach');
			$data['apply_ach'] = $this->config->get('payment_chargeanywhere_apply_ach');

			$data['locale'] = preg_replace('/-(.+?)+/', '', $this->config->get('config_language')) . '_' . $country['iso_code_2'];
			$data['currency_value'] = $this->currency->getValue($this->session->data['currency']);
			$data['decimal_place'] = 2;//$setting['currency'][$data['currency_code']]['decimal_place'];
			$data['language'] = $this->config->get('config_language');
				
			$data['order_id'] = $this->session->data['order_id'];
			$order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);
			$data['message_amount'] = number_format($order_info['total'] * $data['currency_value'], $data['decimal_place'], '.', '');
			require_once DIR_EXTENSION . 'chargeanywhere/system/library/chargeanywhere.php';
			return $this->load->view('extension/chargeanywhere/payment/chargeanywhere', $data);
	}

	public function getExtraChargeDetails(): void {
		$this->load->model('checkout/cart');
		$totals = [];
		$taxes = $this->cart->getTaxes();
		$totalTax = 0;
		
		if(count($taxes)>0){
			foreach($taxes as $key=>$tax){
				$totalTax+=$tax;
			}
		}
		
		$total = 0;
		$subtotal = 0;
		($this->model_checkout_cart->getTotals)($totals, $taxes, $total);
		$products = $this->model_checkout_cart->getProducts();
		foreach ($products as $product) {
			$subtotal += $product['total'];
		}
		$json['data'] = $total;

		$_config = new \Opencart\System\Engine\Config();
		$_config->addPath(DIR_EXTENSION . 'chargeanywhere/system/config/');
		$_config->load('chargeanywhere');
		
		$config_setting = $_config->get('chargeanywhere_setting');
	
		$setting = array_replace_recursive((array)$config_setting, (array)$this->config->get('payment_chargeanywhere_setting'));

		$options = array();
		$options['accept_credit'] = $this->config->get('payment_chargeanywhere_accept_credit');
		$options['apply_credit_service'] = $this->config->get('payment_chargeanywhere_apply_credit');
		$options['credit_service_fee_value'] = $this->config->get('payment_chargeanywhere_credit_fee_value');
		$options['credit_service_fee'] = $this->config->get('payment_chargeanywhere_credit_service');
		$options['credit_service_fee_type'] = $this->config->get('payment_chargeanywhere_credit_fee_service');

		$options['accept_ach'] = $this->config->get('payment_chargeanywhere_accept_ach');
		$options['apply_ach_service'] = $this->config->get('payment_chargeanywhere_apply_ach');
		$options['ach_service_fee_value'] = $this->config->get('payment_chargeanywhere_ach_fee_value');
		$options['ach_service_fee'] = $this->config->get('payment_chargeanywhere_ach_service');
		$options['ach_service_fee_type'] = $this->config->get('payment_chargeanywhere_ach_fee_service');
		$options['apply_credit_tax'] = $this->config->get('payment_chargeanywhere_credit_apply_tax');
		$options['apply_credit_convenience'] = $this->config->get('payment_chargeanywhere_apply_credit_convenience');
		$options['credit_convenience_label'] = $this->config->get('payment_chargeanywhere_credit_convenience_service');
		$options['credit_convenience_fee'] = $this->config->get('payment_chargeanywhere_credit_convenience_fee_value');

		//$options['apply_ach_tax'] = $this->config->get('payment_chargeanywhere_accept_ach');
		
		$options['apply_ach_tax'] = $this->config->get('payment_chargeanywhere_ach_apply_tax');
		$options['apply_ach_convenience'] = $this->config->get('payment_chargeanywhere_apply_ach_convenience');
		$options['ach_convenience_label'] = $this->config->get('payment_chargeanywhere_ach_convenience_service');
		$options['ach_convenience_fee'] = $this->config->get('payment_chargeanywhere_ach_convenience_fee_value');

		$chargeanywhere_option = $this->request->post['chargeanywhere_option'];
		$chargeanywhere_fee_options = array();
		$chargeanywhere_tax = 0;
		if(isset($chargeanywhere_option) && $chargeanywhere_option == 'credit'){
			if($options['accept_credit'] && $options['apply_credit_service'] && $options['credit_service_fee_value']>0){
			    $percentage = $options['credit_service_fee_value'] / 100;  // Percentage (5%) in float
				if($options['apply_credit_tax'])
					$percentage_fee = ($subtotal + $totalTax) * $percentage;
				else
					$percentage_fee = ($subtotal) * $percentage;

				$percentage_fee_text = number_format((float)$percentage_fee, 2, '.', '');
				$totals[] = array("extension" => "chargeanywhere","code" => "tax","title" => (trim($options['credit_service_fee']) != '' ? $options['credit_service_fee'] : 'Service Fee'),"value" => $percentage_fee_text,"name"=>"credit_service_fee","sort_order" => 6);
			}
			if($options['accept_credit'] && $options['apply_credit_convenience'] && $options['credit_convenience_fee']>0){
				$convfee_format = number_format((float)$options['credit_convenience_fee'], 2, '.', ''); 
				$totals[] = array("extension" => "chargeanywhere","code" => "tax","title" => (trim($options['credit_convenience_label']) != '' ? $options['credit_convenience_label'] : 'Convenience Fee'),"value" => $convfee_format,"name"=>"credit_convenience_fee","sort_order" => 7);
			}
		}
		
		if(isset($chargeanywhere_option) && $chargeanywhere_option == 'ach'){
			if($options['accept_ach']  && $options['apply_ach_service'] && $options['ach_service_fee_value']>0){
				$percentage = $options['ach_service_fee_value'] / 100;  // Percentage (5%) in float
				if($options['apply_ach_tax'])
					$percentage_fee = ($subtotal + $totalTax) * $percentage;
				else
					$percentage_fee = ($subtotal) * $percentage;
					$percentage_fee_text = number_format((float)$percentage_fee, 2, '.', '');
				$totals[] = array("extension" => "chargeanywhere","code" => "tax","title" => (trim($options['ach_service_fee']) != '' ? $options['ach_service_fee'] : 'Service Fee' ),"name"=>"ach_service_fee","value" => $percentage_fee_text,"sort_order" => 6);
			}
			if($options['accept_ach'] && $options['apply_ach_convenience'] && $options['ach_convenience_fee']>0){
				$convfee_format = number_format((float)$options['ach_convenience_fee'], 2, '.', ''); 
				$totals[] = array("extension" => "chargeanywhere","code" => "tax","title" => (trim($options['ach_convenience_label']) != ''? $options['ach_convenience_label']: 'Convenience Fee'),"value" => $convfee_format ,"name"=>"ach_convenience_label","sort_order" => 7);
			}
		}

		$sort_order = [];
		
		foreach ($totals as $key => $value) {
			if($value['code'] == 'total'){
				$totals[$key]['value'] += $chargeanywhere_tax;
			}
			$sort_order[$key] = $value['sort_order'];
		}
		array_multisort($sort_order, SORT_ASC, $totals); 
		$json['totals'] =  $totals;
		$json['success'] = $this->language->get('text_success');
		$this->response->addHeader('Content-Type: application/json');
		$this->response->setOutput(json_encode($json));			
	}
	
	public function addHistory(): void {
		$this->load->language('api/sale/order');

		$json = [];

		// Add keys for missing post vars
		$keys = [
			'order_id',
			'order_status_id',
			'comment',
			'notify',
			'override'
		];

		foreach ($keys as $key) {
			if (!isset($this->request->post[$key])) {
				$this->request->post[$key] = '';
			}
		}

		$this->load->model('checkout/order');

		$order_info = $this->model_checkout_order->getOrder((int)$this->request->post['order_id']);
		$cfee_details = (array)$order_info['payment_custom_field'];

		$fee_details = htmlspecialchars_decode($order_info['payment_custom_field']['chargeanywhere_fee_details']);
		$fee_details = (array) json_decode($fee_details);

		if (!$order_info) {
			$json['error'] = $this->language->get('error_order');
		}
		$environment = $this->config->get('payment_chargeanywhere_environment');
		
		if (!$json) {
			$data = (array)$order_info['payment_custom_field'];
			
			$ordStatus = $this->config->get('payment_chargeanywhere_setting');

			if((int)$this->request->post['order_status_id'] == $ordStatus['order_status']['completed']['id'] && $order_info['payment_code'] == 'chargeanywhere' &&  $cfee_details['TransactionType'] == 'Force'){
				$_config = new \Opencart\System\Engine\Config();
				$_config->addPath(DIR_EXTENSION . 'chargeanywhere/system/config/');
				$_config->load('chargeanywhere');
				$config_setting = $_config->get('chargeanywhere_setting');
				
				$setting = array_replace_recursive((array)$config_setting, (array)$this->config->get('payment_chargeanywhere_setting'));
				
				$terminal_id = $this->config->get('payment_chargeanywhere_terminal_id');
				$secret = $this->config->get('payment_chargeanywhere_secret');
				$merchant_id = $this->config->get('payment_chargeanywhere_merchant_id');

				$credit_refund_ser_fee = $this->config->get('payment_chargeanywhere_credit_refund');
				$credit_refund_con_fee = $this->config->get('payment_chargeanywhere_apply_credit_convenience_refund');

				$ach_refund_ser_fee = $this->config->get('payment_chargeanywhere_ach_refund');
				$ach_refund_con_fee = $this->config->get('payment_chargeanywhere_apply_ach_convenience_refund');

				$data['InvoiceNumber'] = $this->request->post['order_id'];
				$data['MerchantId'] = $merchant_id;
				$data['TerminalId'] = $terminal_id;
				$data['Secret'] = $secret;
				//$data['Amount'] = $order_info['total'];

				$tax_total = 0;
				if($cfee_details['payment_option'] === "ach"){
					if(isset($fee_details['ach_service_fee']))
						$tax_total += $fee_details['ach_service_fee'];
					if(isset($fee_details['ach_convenience_service_fee']))
						$tax_total += $fee_details['ach_convenience_service_fee'];
				}

				if($cfee_details['payment_option'] === "credit"){
					if(isset($fee_details['credit_service_fee']))
						$tax_total += $fee_details['credit_service_fee'];
					if(isset($fee_details['credit_convenience_service_fee']))
						$tax_total += $fee_details['credit_convenience_service_fee'];
				}
				$data['Amount'] = $order_info['total'] + $tax_total;
				$data['ServiceFeeLabel'] = $cfee_details['ServiceFeeLabel'];
				$data['ServiceFeeAmount'] = $tax_total;
				$data['ServiceFeeProvided'] = 1;

				$dataOBJ = json_encode($data);
				$log = $this->config->get('payment_chargeanywhere_log');
				if($log == 1){
					$this->logData("Request", array(
						'merchant_id'  => $merchant_id,
						'terminal_id'  => $terminal_id,
						'invoiceNumber' => $this->request->post['order_id'],
						'customernumber' => $order_info['customer_id'], 
						'amount' => $data['Amount'],
						'transactionType' => $data['TransactionType'],
						'ServiceFeeLabel' => $data['ServiceFeeLabel'],
               			'ServiceFeeAmount' => $data['ServiceFeeAmount'],
               			'OriginalTransactionType' => $data['OriginalTransactionType'],
					));
				}

				$response_data = $this->captureTransaction($environment,$dataOBJ);
				
				$responsedata = explode("&",$response_data);
				
				$response = array();
				foreach($responsedata as $key=>$val){
					$txt = explode("=",$val);
					$response[$txt[0]] = $txt[1];
				}

				if($log == 1){
					$this->logData("Response", array(
						'merchant_id'  => $merchant_id,
						'terminal_id'  => $terminal_id,
						'invoicenumber'=> $this->request->post['order_id'], 
               			'responsecode' => $response['ResponseCode'],
               			'approvalcode' => $response['ApprovalCode'],
               			'AuthorizedAmount' => $response['AuthorizedAmount'],
               			'ReferenceNumber' => $response['ReferenceNumber'],
               			'transactionType' => $data['TransactionType'],
					));
				}

				if(isset($response['ResponseCode']) && $response['ResponseCode'] == '000'){
					$this->load->model('extension/chargeanywhere/payment/chargeanywhere');
					$cfee_details['TransactionType'] = 'Return';
					$cfee_details['OriginalTransactionType'] = 'Sale';
					$this->model_extension_chargeanywhere_payment_chargeanywhere->updatePaymentCustomDetails($this->request->post['order_id'], json_encode($cfee_details));
		
					$this->model_checkout_order->addHistory((int)$this->request->post['order_id'], $ordStatus['order_status']['completed']['id']);
				} else {
					$json['error'] = $responsedata;
				}
			} else if((int)$this->request->post['order_status_id'] == $ordStatus['order_status']['refunded']['id'] && $order_info['payment_code'] == 'chargeanywhere' && $cfee_details['TransactionType'] == 'Return'){
				$_config = new \Opencart\System\Engine\Config();
				$_config->addPath(DIR_EXTENSION . 'chargeanywhere/system/config/');
				$_config->load('chargeanywhere');
				$config_setting = $_config->get('chargeanywhere_setting');
				
				$setting = array_replace_recursive((array)$config_setting, (array)$this->config->get('payment_chargeanywhere_setting'));
				
				$terminal_id = $this->config->get('payment_chargeanywhere_terminal_id');
				$secret = $this->config->get('payment_chargeanywhere_secret');
				$merchant_id = $this->config->get('payment_chargeanywhere_merchant_id');
				
				$credit_refund_ser_fee = $this->config->get('payment_chargeanywhere_credit_refund');
				$credit_refund_con_fee = $this->config->get('payment_chargeanywhere_apply_credit_convenience_refund');

				$ach_refund_ser_fee = $this->config->get('payment_chargeanywhere_ach_refund');
				$ach_refund_con_fee = $this->config->get('payment_chargeanywhere_apply_ach_convenience_refund');
				
				$data['MerchantId'] = $merchant_id;
				$data['TerminalId'] = $terminal_id;
				$data['Secret'] = $secret;
				$data['TransactionType'] =  'Return';
				$data['OriginalTransactionType'] =  'Sale';
				$data['CustomerNumber'] = $order_info['customer_id'];
				$data['InvoiceNumber'] = $this->request->post['order_id'];
				$ser_total = 0;
				$con_total = 0;

				if($cfee_details['payment_option'] === "credit"){
					if(isset($fee_details['credit_service_fee']) && $credit_refund_ser_fee)
						$ser_total += $fee_details['credit_service_fee'];
					if(isset($fee_details['credit_convenience_service_fee']) && $credit_refund_con_fee)
						$con_total += $fee_details['credit_convenience_service_fee'];
				}
				
				if($cfee_details['payment_option'] === "ach"){
					if(isset($fee_details['ach_service_fee']) && $ach_refund_ser_fee)
						$ser_total += $fee_details['ach_service_fee'];
					if(isset($fee_details['ach_convenience_service_fee']) && $ach_refund_con_fee)
						$con_total += $fee_details['ach_convenience_service_fee'];
				}
				
				$data['Amount'] = (float) $order_info['total'] + $ser_total + $con_total;
				$data['ServiceFeeLabel'] = $cfee_details['ServiceFeeLabel'];
				$data['ServiceFeeAmount'] = $ser_total + $con_total;
				$data['ServiceFeeProvided'] = 1;

				$data['EmailCustomer'] =  'false';
				if($this->config->get('payment_chargeanywhere_email_customer'))
					$data['EmailCustomer'] =  'true';
				
				$data['EmailMerchant'] =  'false';	
				if($this->config->get('payment_chargeanywhere_email_merchant'))
					$data['EmailMerchant'] =  'true';
				$dataVal = json_encode($data);
				//log file
				$log = $this->config->get('payment_chargeanywhere_log');
				if($log == 1){
					$this->logData("Request", array(
						'merchant_id'  => $merchant_id,
						'terminal_id'  => $terminal_id,
						'invoiceNumber' => $this->request->post['order_id'],
						'customernumber' => $data['CustomerNumber'], 
						'amount' => $data['Amount'],
						'transactionType' => $data['TransactionType'],
						'ServiceFeeLabel' => $data['ServiceFeeLabel'],
               			'ServiceFeeAmount' => $data['ServiceFeeAmount'],
               			'OriginalTransactionType' => $data['OriginalTransactionType'],
					));
				}

				$response_data = $this->captureTransaction($environment, $dataVal);
				$responsedata = explode("&",$response_data);
				$response = array();
				foreach($responsedata as $key=>$val){
					$txt = explode("=",$val);
					$response[$txt[0]] = $txt[1];
				}
				/*if($response['ResponseCode'] == '000'){
					$this->model_checkout_order->addHistory((int)$this->request->post['order_id'], 15);
				}*/

				if($log == 1){
					$this->logData("Response", array(
						'merchant_id'  => $merchant_id,
						'terminal_id'  => $terminal_id,
						'invoicenumber'=> $this->request->post['order_id'], 
               			'responsecode' => $response['ResponseCode'],
               			'approvalcode' => $response['ApprovalCode'],
               			'AuthorizedAmount' => $response['AuthorizedAmount'],
               			'ReferenceNumber' => $response['ReferenceNumber'],
               			'transactionType' => $data['TransactionType'],
					));
				}
				if(isset($response['ResponseCode']) && $response['ResponseCode'] == '000'){
					$this->load->model('extension/chargeanywhere/payment/chargeanywhere');
					$cfee_details['TransactionType'] = 'complete';
					$this->model_extension_chargeanywhere_payment_chargeanywhere->updatePaymentCustomDetails($this->request->post['order_id'], json_encode($cfee_details));
		
					$this->model_checkout_order->addHistory((int)$this->request->post['order_id'], $ordStatus['order_status']['refunded']['id']);
				} else {
					$json['error'] = $responsedata;
				}
			}
			else{
				$this->model_checkout_order->addHistory((int)$this->request->post['order_id'], (int)$this->request->post['order_status_id'], (string)$this->request->post['comment'], (bool)$this->request->post['notify'], (bool)$this->request->post['override']);
			}
			$json['success'] = $this->language->get('text_success');
		}

		$this->response->addHeader('Content-Type: application/json');
		$this->response->setOutput(json_encode($json));
	}
	public function seed_gen(){
		return rand(1,9).time();
	 }
	public function paymentResponse() {
		$this->load->model('checkout/order');
		$order_data = [];

		$log = $this->config->get('payment_chargeanywhere_log');
		if($log == 1){
			$this->logData("Response", array(
				'merchant_id'  => $this->config->get('payment_chargeanywhere_merchant_id'),
				'terminal_id'  => $this->config->get('payment_chargeanywhere_terminal_id'),
				'invoicenumber'=> $this->request->post['InvoiceNumber'], 
				'responsecode' => $this->request->post['ResponseCode'],
				'approvalcode' => $this->request->post['ApprovalCode'],
				'chargeanywhere_option' => $this->request->post['payment_option'],
				'transaction_type' => $this->config->get('payment_chargeanywhere_transaction_method'),
			));
		}

		if($this->request->post['ResponseCode'] == '000'){
			$transaction_id = $this->request->post['ApprovalCode']."_".$this->request->post['ReferenceNumber'];
			$order_info = $this->model_checkout_order->getOrder($this->request->post['InvoiceNumber']);
			$_config = new \Opencart\System\Engine\Config();
			$_config->addPath(DIR_EXTENSION . 'chargeanywhere/system/config/');
			$_config->load('chargeanywhere');
			$config_setting = $_config->get('chargeanywhere_setting');
			$setting = array_replace_recursive((array)$config_setting, (array)$this->config->get('payment_chargeanywhere_setting'));
				
			$terminal_id = $this->config->get('payment_chargeanywhere_terminal_id');
			$secret = $this->config->get('payment_chargeanywhere_secret');
			$merchant_id = $this->config->get('payment_chargeanywhere_merchant_id');
			$environment = $this->config->get('payment_chargeanywhere_environment');
			$transaction_mode = $this->config->get('payment_chargeanywhere_transaction_method');
			
			$authorize_args['MerchantId'] = $merchant_id;
			$authorize_args['TerminalId'] = $terminal_id;
			$authorize_args['Seed'] = $this->seed_gen();
			$transaction_method = $this->config->get('payment_chargeanywhere_transaction_method');
			$chargeanywhere_fee_details = isset( $this->request->post['chargeanywhere_fee_details'] ) ? $this->request->post['chargeanywhere_fee_details'] : '';
			$tType = "";
			if($this->request->post['TransactionType'] == 'Sale')
				$tType = "Return";

			if($this->request->post['TransactionType'] == 'Auth Only')
				$tType = "Force";
			$data = array(
				'CustomerNumber'=> $this->request->post['CustomerNumber'],
				'Seed' => $authorize_args['Seed'],
				'Mode' => '2',
				'Version' => '2.0',
				'TransactionType' => $tType,
				'OriginalTransactionType' => $this->request->post['TransactionType'],
				'OriginalApprovalCode' => $this->request->post['ApprovalCode'],
				'OriginalReferenceNumber' =>  $this->request->post['ReferenceNumber'],
				'OriginalAmount' => $this->request->post['AuthorizedAmount'],
				'Amount' => $order_info['total'],
				'Tax'    => $this->request->post['Tax'],
				'Shipping'    => $this->request->post['Shipping'],
				'paymentMethod'=>$this->request->post['PaymentMethod'],
				'payment_option'=>$this->request->post['payment_option'],
				'ServiceFeeLabel' => $this->request->post['ServiceFeeLabel'],
               	'ServiceFeeAmount' => $this->request->post['ServiceFeeAmount'],
               	'ServiceFeeProvided' => 1,
				'chargeanywhere_fee_details' =>$chargeanywhere_fee_details
			);
			
			$data = json_encode($data);
			//$order_info['payment_custom_field'] = $data;
			$this->load->model('extension/chargeanywhere/payment/chargeanywhere');
			$this->model_extension_chargeanywhere_payment_chargeanywhere->updatePaymentCustomDetails($this->request->post['InvoiceNumber'],$data);
			$this->model_checkout_order->editTransactionId($this->request->post['InvoiceNumber'], $transaction_id);
			if ($order_info && $this->request->post['TransactionType'] == 'Sale') {
				$this->model_checkout_order->addHistory($this->request->post['InvoiceNumber'], 15);
			}
			//$this->model_checkout_order->editOrder($this->request->post['InvoiceNumber'], $order_info);
			$this->response->redirect($this->url->link('checkout/success', '', 'SSL'));
		} else{
			$this->response->redirect($this->url->link('checkout/failure', '', 'SSL'));
		}
		//echo $this->session->data['order_id'];
		
		//$this->response->redirect($this->url->link('checkout/success', '', 'SSL'));
	}

	public function captureTransaction($environment,$data){
		$vPostData = "";
		$data = json_decode($data);
		foreach($data as $key => $val){
			if($vPostData != "") $vPostData .= "&";
			$vPostData .= $key."=".$val;
		}
		$curl = curl_init();
		curl_setopt_array($curl, array(
		CURLOPT_URL => $this->server_capture_refund[$environment],
		CURLOPT_RETURNTRANSFER => true,
		CURLOPT_ENCODING => '',
		CURLOPT_MAXREDIRS => 10,
		CURLOPT_TIMEOUT => 0,
		CURLOPT_FOLLOWLOCATION => true,
		CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
		CURLOPT_CUSTOMREQUEST => 'POST',
		CURLOPT_POSTFIELDS => $vPostData,
		CURLOPT_HTTPHEADER => array(
			'Content-Type: application/x-www-form-urlencoded'
		),
		));

		$response = curl_exec($curl);
		curl_close($curl);
		return $response;
	}
		
	public function createOrder(): void {		
		
		$this->load->language('api/sale/order');
		$card_number = '';
		$cvv = '';
		$exp_month = '';
		$exp_year = '';
		$exp_date = '';
		$credit_service_fee = 0;
		$credit_convenience_fee = 0;
		$ach_service_fee = 0;
		$ach_convenience_fee = 0;

		$tax_total = 0;
		$chargeanywhere_option = $this->request->post['chargeanywhere-option'];
		if(isset($this->request->post['chargeanywhere-card-number']))
			$card_number = str_replace(" ", "", $this->request->post['chargeanywhere-card-number']);
		if(isset($this->request->post['chargeanywhere-card-cvc']))
			$cvv = $this->request->post['chargeanywhere-card-cvc'];
		if(isset($this->request->post['chargeanywhere-card-expiry'])){
			$exp_date_array = explode( "/", $this->request->post['chargeanywhere-card-expiry'] );
        	$exp_month = trim( $exp_date_array[0] );
       	 	$exp_year = trim( $exp_date_array[1] );
        	$exp_date = $exp_month . substr( $exp_year, -2 );
		}
		$fee_store_details = array();
		$service_fee = 0;
		$conv_service_fee = 0;
		if(isset($this->request->post['chargeanywhere-credit-service-fee'])){
			$credit_service_fee = $this->request->post['chargeanywhere-credit-service-fee'];
			$service_fee = $credit_service_fee;
			$fee_store_details['credit_service_fee'] = $credit_service_fee;
			$tax_total += $credit_service_fee;
		}
		$convenience_fee = 0;
		if(isset($this->request->post['chargeanywhere-credit-convenience-fee'])){
			$credit_convenience_fee = $this->request->post['chargeanywhere-credit-convenience-fee'];
			$conv_service_fee = $credit_convenience_fee;
			$fee_store_details['credit_convenience_service_fee'] = $credit_convenience_fee;
			//$tax_total += $credit_convenience_fee;
			$convenience_fee = $credit_convenience_fee;
		}
		
		if(isset($this->request->post['chargeanywhere-ach-service-fee'])){
			$ach_service_fee = $this->request->post['chargeanywhere-ach-service-fee'];
			$service_fee = $ach_service_fee;
			$fee_store_details['ach_service_fee'] = $ach_service_fee;
			$tax_total += $ach_service_fee;
		}
		if(isset($this->request->post['chargeanywhere-ach-convenience-fee'])){
			$ach_convenience_fee = $this->request->post['chargeanywhere-ach-convenience-fee'];
			$conv_service_fee = $ach_convenience_fee;
			$fee_store_details['ach_convenience_service_fee'] = $ach_convenience_fee;
			//$tax_total += $ach_convenience_fee;
			$convenience_fee = $ach_convenience_fee;
		}

		$account_number = '';
		$routing_number = '';
		$acctype = '';
		if(isset($this->request->post['chargeanywhere-account-number']))
			$account_number = str_replace(" ", "", $this->request->post['chargeanywhere-account-number']);
		if(isset($this->request->post['chargeanywhere-routing-number']))
			$routing_number = str_replace(" ", "", $this->request->post['chargeanywhere-routing-number']);
		if(isset($this->request->post['chargeanywhere-acctype']))
			$acctype = $this->request->post['chargeanywhere-acctype'];
		$_config = new \Opencart\System\Engine\Config();
		$_config->addPath(DIR_EXTENSION . 'chargeanywhere/system/config/');
		$_config->load('chargeanywhere');
			
		$config_setting = $_config->get('chargeanywhere_setting');
		
		$setting = array_replace_recursive((array)$config_setting, (array)$this->config->get('payment_chargeanywhere_setting'));
		
		$terminal_id = $this->config->get('payment_chargeanywhere_terminal_id');
		$secret = $this->config->get('payment_chargeanywhere_secret');
		$merchant_id = $this->config->get('payment_chargeanywhere_merchant_id');
		$environment = $this->config->get('payment_chargeanywhere_environment');
		$transaction_mode = $this->config->get('payment_chargeanywhere_transaction_method');
		$vSeed = $this->seed_gen();
		$authorize_args['MerchantId'] = $merchant_id;
        $authorize_args['TerminalId'] = $terminal_id;
        $authorize_args['Seed'] = $vSeed;
		$transaction_method = $this->config->get('payment_chargeanywhere_transaction_method');



		$json = [];

		// Validate cart has products and has stock.
		if (($this->cart->hasProducts() || !empty($this->session->data['vouchers']))) {
			if (!$this->cart->hasStock() && !$this->config->get('config_stock_checkout')) {
				$json['error']['stock'] = $this->language->get('error_stock');
			}
		} else {
			$json['error']['product'] = $this->language->get('error_product');
		}

		// Validate minimum quantity requirements.
		$products = $this->cart->getProducts();

		foreach ($products as $product) {
			if (!$product['minimum']) {
				$json['error']['minimum'] = sprintf($this->language->get('error_minimum'), $product['name'], $product['minimum']);

				break;
			}
		}

		// Customer
		if (!isset($this->session->data['customer'])) {
			$json['error']['customer'] = $this->language->get('error_customer');
		}

		// Payment Address
		// if ($this->config->get('config_checkout_address') && !isset($this->session->data['payment_address'])) {
		// 	$json['error']['payment_address'] = $this->language->get('error_payment_address');
		// }

		// Shipping
		if ($this->cart->hasShipping()) {
			// Shipping Address
			if (!isset($this->session->data['shipping_address'])) {
				$json['error']['shipping_address'] = $this->language->get('error_shipping_address');
			}

			// Validate shipping method
			if (isset($this->session->data['shipping_method']) && isset($this->session->data['shipping_methods'])) {
				$shipping = explode('.', $this->session->data['shipping_method']);

				if (!isset($shipping[0]) || !isset($shipping[1]) || !isset($this->session->data['shipping_methods'][$shipping[0]]['quote'][$shipping[1]])) {
					$json['error']['shipping_method'] = $this->language->get('error_shipping_method');
				}
			} else {
				$json['error']['shipping_method'] = $this->language->get('error_shipping_method');
			}
		} else {
			unset($this->session->data['shipping_address']);
			unset($this->session->data['shipping_method']);
			unset($this->session->data['shipping_methods']);
		}

		// Payment Method
		if (!isset($this->session->data['payment_method']) || !isset($this->session->data['payment_methods']) || !isset($this->session->data['payment_methods'][$this->session->data['payment_method']])) {
			$json['error']['payment_method'] = $this->language->get('error_payment_method');
		}

		if (!$json) {
			$order_data = [];

			// Store Details
			$order_data['invoice_prefix'] = $this->config->get('config_invoice_prefix');

			$order_data['store_id'] = $this->config->get('config_store_id');
			$order_data['store_name'] = $this->config->get('config_name');
			$order_data['store_url'] = $this->config->get('config_url');

			// Customer Details
			$order_data['customer_id'] = $this->session->data['customer']['customer_id'];
			$order_data['customer_group_id'] = $this->session->data['customer']['customer_group_id'];
			$order_data['firstname'] = $this->session->data['customer']['firstname'];
			$order_data['lastname'] = $this->session->data['customer']['lastname'];
			$order_data['email'] = $this->session->data['customer']['email'];
			$order_data['telephone'] = $this->session->data['customer']['telephone'];
			$order_data['custom_field'] = $this->session->data['customer']['custom_field'];

			$authorize_args['CustomerNumber'] = $order_data['customer_id'];
        
			// Payment Details
			if ($this->config->get('config_checkout_address')) {
				$order_data['payment_firstname'] = $this->session->data['payment_address']['firstname'];
				$order_data['payment_lastname'] = $this->session->data['payment_address']['lastname'];
				$order_data['payment_company'] = $this->session->data['payment_address']['company'];
				$order_data['payment_address_1'] = $this->session->data['payment_address']['address_1'];
				$order_data['payment_address_2'] = $this->session->data['payment_address']['address_2'];
				$order_data['payment_city'] = $this->session->data['payment_address']['city'];
				$order_data['payment_postcode'] = $this->session->data['payment_address']['postcode'];
				$order_data['payment_zone'] = $this->session->data['payment_address']['zone'];
				$order_data['payment_zone_id'] = $this->session->data['payment_address']['zone_id'];
				$order_data['payment_country'] = $this->session->data['payment_address']['country'];
				$order_data['payment_country_id'] = $this->session->data['payment_address']['country_id'];
				$order_data['payment_address_format'] = $this->session->data['payment_address']['address_format'];
				$order_data['payment_custom_field'] = isset($this->session->data['payment_address']['custom_field']) ? $this->session->data['payment_address']['custom_field'] : [];


				$authorize_args['BIFirstName'] = $this->session->data['payment_address']['firstname'];
				$authorize_args['BILastName'] = $this->session->data['payment_address']['lastname'];
				$authorize_args['BIAddress'] = $this->session->data['payment_address']['address_1'] .' '. $this->session->data['payment_address']['address_2'];
				$authorize_args['BICity'] =$this->session->data['payment_address']['city'];
				$authorize_args['BIState'] = $this->session->data['payment_address']['zone'];
				$authorize_args['BIZipCode'] = $this->session->data['payment_address']['postcode'];
				$authorize_args['BICountry'] = $this->session->data['payment_address']['country'];
				$authorize_args['BIEmail'] = $order_data['email'];
	   
			} else {
				$order_data['payment_firstname'] = '';
				$order_data['payment_lastname'] = '';
				$order_data['payment_company'] = '';
				$order_data['payment_address_1'] = '';
				$order_data['payment_address_2'] = '';
				$order_data['payment_city'] = '';
				$order_data['payment_postcode'] = '';
				$order_data['payment_zone'] = '';
				$order_data['payment_zone_id'] = 0;
				$order_data['payment_country'] = '';
				$order_data['payment_country_id'] = 0;
				$order_data['payment_address_format'] = '';
				$order_data['payment_custom_field'] = [];
			}

			$payment_method_info = $this->session->data['payment_methods'][$this->session->data['payment_method']];

			$order_data['payment_method'] = $payment_method_info['title'];
			$order_data['payment_code'] = $payment_method_info['code'];

			// Shipping Details
			if ($this->cart->hasShipping()) {
				$order_data['shipping_firstname'] = $this->session->data['shipping_address']['firstname'];
				$order_data['shipping_lastname'] = $this->session->data['shipping_address']['lastname'];
				$order_data['shipping_company'] = $this->session->data['shipping_address']['company'];
				$order_data['shipping_address_1'] = $this->session->data['shipping_address']['address_1'];
				$order_data['shipping_address_2'] = $this->session->data['shipping_address']['address_2'];
				$order_data['shipping_city'] = $this->session->data['shipping_address']['city'];
				$order_data['shipping_postcode'] = $this->session->data['shipping_address']['postcode'];
				$order_data['shipping_zone'] = $this->session->data['shipping_address']['zone'];
				$order_data['shipping_zone_id'] = $this->session->data['shipping_address']['zone_id'];
				$order_data['shipping_country'] = $this->session->data['shipping_address']['country'];
				$order_data['shipping_country_id'] = $this->session->data['shipping_address']['country_id'];
				$order_data['shipping_address_format'] = $this->session->data['shipping_address']['address_format'];
				$order_data['shipping_custom_field'] = isset($this->session->data['shipping_address']['custom_field']) ? $this->session->data['shipping_address']['custom_field'] : [];
			} else {
				$order_data['shipping_firstname'] = '';
				$order_data['shipping_lastname'] = '';
				$order_data['shipping_company'] = '';
				$order_data['shipping_address_1'] = '';
				$order_data['shipping_address_2'] = '';
				$order_data['shipping_city'] = '';
				$order_data['shipping_postcode'] = '';
				$order_data['shipping_zone'] = '';
				$order_data['shipping_zone_id'] = '';
				$order_data['shipping_country'] = '';
				$order_data['shipping_country_id'] = '';
				$order_data['shipping_address_format'] = '';
				$order_data['shipping_custom_field'] = [];
			}

			if (isset($this->session->data['shipping_method'])) {
				$shipping = explode('.', $this->session->data['shipping_method']);

				$shipping_method_info = $this->session->data['shipping_methods'][$shipping[0]]['quote'][$shipping[1]];

				$order_data['shipping_method'] = $shipping_method_info['title'];
				$order_data['shipping_code'] = $shipping_method_info['code'];
			} else {
				$order_data['shipping_method'] = '';
				$order_data['shipping_code'] = '';
			}

			$points = 0;

			// Products
			$order_data['products'] = [];

			foreach ($this->cart->getProducts() as $product) {
				$option_data = [];

				foreach ($product['option'] as $option) {
					$option_data[] = [
						'product_option_id'       => $option['product_option_id'],
						'product_option_value_id' => $option['product_option_value_id'],
						'option_id'               => $option['option_id'],
						'option_value_id'         => $option['option_value_id'],
						'name'                    => $option['name'],
						'value'                   => $option['value'],
						'type'                    => $option['type']
					];
				}

				$order_data['products'][] = [
					'product_id'   => $product['product_id'],
					'master_id'    => $product['master_id'],
					'name'         => $product['name'],
					'model'        => $product['model'],
					'option'       => $option_data,
					'subscription' => $product['subscription'],
					'download'     => $product['download'],
					'quantity'     => $product['quantity'],
					'subtract'     => $product['subtract'],
					'price'        => $product['price'],
					'total'        => $product['total'],
					'tax'          => $this->tax->getTax($product['price'], $product['tax_class_id']),
					'reward'       => $product['reward']
				];

				$points += $product['reward'];
			}

			// Gift Voucher
			$order_data['vouchers'] = [];

			if (!empty($this->session->data['vouchers'])) {
				foreach ($this->session->data['vouchers'] as $voucher) {
					$order_data['vouchers'][] = [
						'description'      => $voucher['description'],
						'code'             => Helper\General\token(10),
						'to_name'          => $voucher['to_name'],
						'to_email'         => $voucher['to_email'],
						'from_name'        => $voucher['from_name'],
						'from_email'       => $voucher['from_email'],
						'voucher_theme_id' => $voucher['voucher_theme_id'],
						'message'          => $voucher['message'],
						'amount'           => $voucher['amount']
					];
				}
			}

			if (isset($this->session->data['comment'])) {
				$order_data['comment'] = $this->session->data['comment'];
			} else {
				$order_data['comment'] = '';
			}

			// Order Totals
			$totals = [];
			$taxes = $this->cart->getTaxes();
			$total = 0;

			$this->load->model('checkout/cart');

			($this->model_checkout_cart->getTotals)($totals, $taxes, $total);
			
			$total_data = [
				'totals' => $totals,
				'taxes'  => $taxes,
				'total'  => $total
			];
			//print_r($totals);
			$totalLength = count($totals) -1;
			$totalObj = $totals[$totalLength];
			unset($totals[$totalLength]);
			
			if($service_fee > 0){
				$totals[$totalLength] = array('extension'=>'opencart','code'=>'tax','title'=>$this->request->post['chargeanywhere-service-fee-label'],'value'=>$service_fee,'sort_order'=>6);
				$totalLength++;
			}
			if($conv_service_fee > 0){
				$totals[$totalLength] = array('extension'=>'opencart','code'=>'tax','title'=>$this->request->post['chargeanywhere-convenience-fee-label'],'value'=>$conv_service_fee,'sort_order'=>7);
				$totalLength++;
			}
			$totalObj['value'] += $service_fee + $conv_service_fee;
			$totalObj['sort_order'] = 50;
			$totals[$totalLength] = $totalObj;
			// // print_r($totals);
			// // exit;
			// //exit;
			// $totals[3] = $totals[2];
			// $totals[2] = array('extension'=>'chargeanywhere','code'=>'tax','title'=>'Service Fee','value'=>'10','sort_order'=>4);
			
			//shipping value calculate
			$shipping = 0;
			$tax = 0;
			foreach($totals as $ind => $val){
				if($val['code'] == 'tax'){
					$tax += $val['value'];
				} else if($val['code'] == 'shipping'){
					$shipping += $val['value'];
				}
			}

			$total = $total + $convenience_fee;
			$authorize_args['Amount'] = $total;
			$vTextTohash = $merchant_id . ':' . $terminal_id . ':' . $vSeed . ':' . $total;
			$signature = hash_hmac('sha512', $vTextTohash, hex2bin($secret));
			$authorize_args['Tax'] = $tax - $service_fee - $convenience_fee;
			$authorize_args['Shipping'] = $shipping;

			$authorize_args['Signature'] = $signature;
			$authorize_args['MerchantReceiptURL'] = $this->url->link('extension/chargeanywhere/payment/chargeanywhere|paymentResponse', '', 'SSL');//"http://localhost/opencart/upload/index.php?route=extension/chargeanywhere/payment/chargeanywhere|paymentResponse";
			
			$authorize_args['Version'] = "2.0";
			$authorize_args['payment_option'] =  $chargeanywhere_option;
			$fee_label = "Service Fee";

			if($chargeanywhere_option == 'credit'){
				if($this->config->get('payment_chargeanywhere_credit_service') != ""){
					$fee_label = $this->config->get('payment_chargeanywhere_credit_service');
				}
				$authorize_args['TransactionType'] = 'Sale';
				if($transaction_mode == 'authorize')
				   $authorize_args['TransactionType'] = 'Auth Only';
				   
				$authorize_args['PaymentMethod'] = 1;
				//$authorize_args['payment_type'] = 'credit';
				$authorize_args['CardNumber'] = $card_number;
				$authorize_args['ExpMonth'] = $exp_month;
				$authorize_args['ExpYear'] = $exp_year;
				$authorize_args['CardVerificationValue'] =  $cvv;
			} else {
				if($this->config->get('payment_chargeanywhere_ach_service') != ""){
					$fee_label = $this->config->get('payment_chargeanywhere_ach_service');
				}
				$authorize_args['PaymentMethod'] = 2;

				//$authorize_args['payment_type'] = 'ach';
				$authorize_args['TransactionType'] = 'Sale';
				$authorize_args['AccountType'] = $acctype;
				$authorize_args['RoutingNumber'] = $routing_number ;
				$authorize_args['AccountNumber'] = $account_number;

				
				$authorize_args['FirstName'] = isset($this->session->data['shipping_address']['firstname']) ? $this->session->data['shipping_address']['firstname']: ''; //$order->get_billing_first_name();
				$authorize_args['LastName'] =  isset($this->session->data['shipping_address']['lastname']) ? $this->session->data['shipping_address']['lastname']: '';
		
			}
			
			$authorize_args['InvoiceNumber'] = $this->session->data['order_id'];
			
			$log = $this->config->get('payment_chargeanywhere_log');
			if($log == 1){
				$this->logData("Request", array(
					'merchant_id'  => $merchant_id,
					'terminal_id'  => $terminal_id,
					'invoiceNumber' => $authorize_args['InvoiceNumber'],
					'customerumber' => $authorize_args['CustomerNumber'], 
					'amount' => $authorize_args['Amount'],
					'transactionType' => $authorize_args['TransactionType'],
					'chargeanywhere_option' => $chargeanywhere_option,
			  ));
			}

			//if($tax_total > 0){
				$authorize_args['ServiceFeeLabel'] = $fee_label;
				$authorize_args['ServiceFeeAmount'] = $tax_total;
				$authorize_args['ServiceFeeProvided'] = 1;
			//}

		//$authorize_args['InvoiceNumber'] = $this->session->data['order_id'];
		$authorize_args['chargeanywhere_fee_details'] = json_encode($fee_store_details);
			$authorize_args['Mode'] = '1';
			$authorize_args['EmailCustomer'] =  'false';
			//$authorize_args['sessionData'] =  json_encode($this->session->data);
			
			if($this->config->get('payment_chargeanywhere_email_customer'))
					$authorize_args['EmailCustomer'] =  'true';
				
				$authorize_args['EmailMerchant'] =  'false';	
				if($this->config->get('payment_chargeanywhere_email_merchant'))
					$authorize_args['EmailMerchant'] =  'true';
			foreach($authorize_args as $key => $value){
				$authorize_args_array[] = "<input type='hidden' name='$key' value='$value'/>";
			}

			
			$order_data['affiliate_id'] = 0;
			$order_data['commission'] = 0;
			$order_data['marketing_id'] = 0;
			$order_data['tracking'] = '';

			if (isset($this->session->data['affiliate_id'])) {
				$subtotal = $this->cart->getSubTotal();

				// Affiliate
				$this->load->model('account/affiliate');

				$affiliate_info = $this->model_account_affiliate->getAffiliate($this->session->data['affiliate_id']);

				if ($affiliate_info) {
					$order_data['affiliate_id'] = $affiliate_info['customer_id'];
					$order_data['commission'] = ($subtotal / 100) * $affiliate_info['commission'];
					$order_data['tracking'] = $affiliate_info['tracking'];
				}
			}

			// We use session to store language code for API access
			$order_data['language_id'] = $this->config->get('config_language_id');
			$order_data['language_code'] = $this->config->get('config_language');

			$order_data['currency_id'] = $this->currency->getId($this->session->data['currency']);
			$order_data['currency_code'] = $this->session->data['currency'];
			$order_data['currency_value'] = $this->currency->getValue($this->session->data['currency']);

			$order_data['ip'] = $this->request->server['REMOTE_ADDR'];

			if (!empty($this->request->server['HTTP_X_FORWARDED_FOR'])) {
				$order_data['forwarded_ip'] = $this->request->server['HTTP_X_FORWARDED_FOR'];
			} elseif (!empty($this->request->server['HTTP_CLIENT_IP'])) {
				$order_data['forwarded_ip'] = $this->request->server['HTTP_CLIENT_IP'];
			} else {
				$order_data['forwarded_ip'] = '';
			}

			if (isset($this->request->server['HTTP_USER_AGENT'])) {
				$order_data['user_agent'] = $this->request->server['HTTP_USER_AGENT'];
			} else {
				$order_data['user_agent'] = '';
			}

			if (isset($this->request->server['HTTP_ACCEPT_LANGUAGE'])) {
				$order_data['accept_language'] = $this->request->server['HTTP_ACCEPT_LANGUAGE'];
			} else {
				$order_data['accept_language'] = '';
			}
			$order_data['totals'] = $totals;
			
			$this->load->model('checkout/order');
			
			if (!isset($this->session->data['order_id'])) {
				$this->session->data['order_id'] = $this->model_checkout_order->addOrder($order_data);
			} else {
				$order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);

				if ($order_info) {
					$this->model_checkout_order->editOrder($this->session->data['order_id'], $order_data);
				}
			}
			
			$json['order_id'] = $this->session->data['order_id'];

			// Set the order history
			if (isset($this->request->post['order_status_id'])) {
				$order_status_id = (int)$this->request->post['order_status_id'];
			} else {
				$order_status_id = $this->config->get('config_order_status_id');
			}
			
			$this->model_checkout_order->addHistory($json['order_id'], $order_status_id);
			
			$json['success'] = $this->language->get('text_success');
			$json['authorize_args'] =  implode('', $authorize_args_array) ;
			$json['points'] = $points;

			if (isset($order_data['affiliate_id'])) {
				$json['commission'] = $this->currency->format($order_data['commission'], $this->config->get('config_currency'));
			}
		}

		$this->response->addHeader('Content-Type: application/json');
		$this->response->setOutput(json_encode($json));
	}

	public function logData($type, $data){
		$date = new \DateTime('now', new \DateTimeZone(date('P')));
		$data = array_merge(array('current_time' => $date->format("Y-m-d H:i:s-u")), $data);
		$fname = date("Y-m-d");
		$fileName = DIR_EXTENSION."chargeanywhere/logs/".$fname.".log";
		$file = fopen($fileName, "a") or die("Unable to open file!");
		fwrite($file, $type.": ".json_encode($data)."\n");
		fclose($file);
	 }
}